<?php
namespace SCM\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Method;
use Util\Validation;
use SCM\Traits\TResponse;

class Efetivos extends \Zeedhi\Framework\Controller\Simple {

    use TResponse;

	protected $efetivos;
    protected $environment;

	public function __construct(\SCM\Services\Efetivos $efetivos, $environment) {
        $this->efetivos = $efetivos;
        $this->environment = $environment;
    }

	public function retornaDatasCardapio(Request $request, Response $response) {
        try {
        	$filter = $request->getFilterCriteria()->getConditions();

            // $nrorg = $filter[0]['value'];
            $nrorg = $this->environment->getNrorg();
            $cdfilial = $filter[1]['value'];
            $dataInicial = $filter[2]['value'];
            $dataFinal = $filter[3]['value'];
            $periodo = $filter[4]['value'];

            $datasCardapio = $this->efetivos->retornaDatasCardapio($nrorg, $cdfilial, $dataInicial, $dataFinal);

			$response->addDataSet(new DataSet('DatasCardapioEfetivos', $datasCardapio));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function retornaTiposCardapio(Request $request, Response $response) {
        try {
            $filter = $request->getFilterCriteria()->getConditions();

            // $nrorg = $filter[0]['value'];
            $nrorg = $this->environment->getNrorg();
            $cdfilial = $filter[1]['value'];
            $dataInicial = $filter[2]['value'];
            $dataFinal = $filter[3]['value'];
            $periodo = $filter[4]['value'];

            $tiposCardapio = $this->efetivos->retornaTiposCardapio($nrorg, $cdfilial, $dataInicial, $dataFinal);

            $response->addDataSet(new DataSet('TiposCardapioEfetivos', $tiposCardapio));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function retornaCardapio(Request $request, Response $response) {
        try {
        	$filter = $request->getFilterCriteria()->getConditions();

            // $nrorg = $filter[0]['value'];
            $nrorg = $this->environment->getNrorg();
            $cdfilial = $filter[1]['value'];
            $dataInicial = $filter[2]['value'];
            $dataFinal = $filter[3]['value'];
            $periodo = $filter[4]['value'];

            $cardapio = $this->efetivos->retornaCardapio($nrorg, $cdfilial, $dataInicial, $dataFinal);

			$response->addDataSet(new DataSet('CardapioEfetivos', $cardapio));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function retornaServicosCardapio(Request $request, Response $response) {
        try {
        // 	$filter = $request->getFilterCriteria()->getConditions();
            $filter = $request->getParameters();
    
            // $nrorg = $filter[0]['value'];
            $nrorg = $this->environment->getNrorg();
            $cdfilial = $filter['CDFILIAL'];
            $dataInicial = $filter['DTINICIAL'];
            $dataFinal = $filter['DTFINAL'];

            $servicosCardapio = $this->efetivos->retornaServicosCardapio($nrorg, $cdfilial, $dataInicial, $dataFinal);

// 			$response->addDataSet(new DataSet('ServicosCardapioEfetivos', $servicosCardapio));
            $this->sendResponse($servicosCardapio);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function retornaClientesDoServico(Request $request, Response $response) {
        try {
            // $filter = $request->getFilterCriteria()->getConditions();
            $filter = $request->getParameters();
            // $nrorg = $filter[0]['value'];
            $nrorg = $this->environment->getNrorg();
            $cdfilial = $filter['CDFILIAL'];
            $dataInicial = $filter['DTINICIAL'];
            $dataFinal = $filter['DTFINAL'];
            $periodo = $filter['PERIODO'];
            $cdservfili = $filter['CDSERVFILI'];
            $cdtipocard = $filter['CDTIPOCARD'];

            $clientesDoServico = $this->efetivos->retornaClientesDoServico($nrorg, $cdfilial, $dataInicial, $dataFinal, $cdservfili, $cdtipocard);

            // $response->addDataSet(new DataSet('ClientesDoServicoEfetivos', $clientesDoServico));
            $this->sendResponse($clientesDoServico);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function update(Request\Dataset $request, Response $response) {
        try {
            $rows = $request->getDataSet()->getRows();
            foreach ($rows as $row) {
                $this->efetivos->update($row);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function updateClientes(Request\Dataset $request, Response $response) {
        try {
            $rows = $request->getDataSet()->getRows();
            foreach ($rows as $row) {
                $this->efetivos->updateClientes($row);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }


}