<?php
/**
 * Created by PhpStorm.
 * User: francismararaujo
 * Date: 23/10/14
 * Time: 17:07
 */

namespace SCM\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;
use SCM\Traits\TResponse;

use SCM\Services\UserService;

class UserController extends \Zeedhi\Framework\Controller\Simple {

    use TResponse;

    protected $userService;
    protected $environment;
    protected $session;

    public function __construct(UserService $userService, $environment, $session) {
        $this->userService = $userService;
        $this->environment = $environment;
        $this->session = $session;
    }

    public function updateUser(Request\Row $request, Response $response) {

        $row = $request->getRow();

        if($row) {

            try{
                $this->userService->updateUser($row);
            } catch(\Exception $e) {
                $response->setError(new Error($e->getMessage(), $e->getCode()));
            };

        }

    }

    public function save(Request\Row $request, Response $response) {

        $user = $request->getRow();

        if($user) {

            try {

                $this->userService->save($user);

            } catch(\Exception $e) {
                $response->setError(new Error($e->getMessage(), $e->getCode()));
            };

        }
    }

    public function retornaFiliaisUsuario(Request\Filter $request, Response $response){
        try {
            // $filter = $request->getFilterCriteria()->getConditions();
            $params = $request->getParameters();
            if ($params) $operador = $params['CDOPERADOR'];
            else $operador = null;
            $unidades = $this->userService->retornaFiliaisUsuario($operador);
            // $response->addDataSet(new DataSet('Units', $unidades));
            $this->sendResponse($unidades);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function filtraFiliaisUsuario(Request\Row $request, Response $response){
        $row = $request->getRow();
        try {
            $operador = $row['CDOPERADOR'];
            $filiais = $row['CDFILIAIS'];
            $unidades = $this->userService->filtraFiliaisUsuario($operador, $filiais);
            $response->addDataSet(new DataSet('Units', $unidades));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function retornaUsuarioAtivo(Request\Filter $request, Response $response) {
        try {
            $filter = $request->getFilterCriteria()->getConditions();
            $dsemailoper = $filter[0]['value'];
            // $nrorg = $filter[1]['value'];
            $nrorg = $this->environment->getNrorg();

            $ativo = $this->userService->retornaUsuarioAtivo($dsemailoper, $nrorg);
            $response->addDataSet(new DataSet('usuario', array('ativo' => $ativo)));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function retornaOperadorAtivo(Request\Filter $request, Response $response) {
        try {
            $filter = $request->getFilterCriteria()->getConditions();
            $cdoperador = $filter[0]['value'];
            $nrorg = $this->environment->getNrorg();
            
            $ativo = $this->userService->retornaOperadorAtivo($cdoperador, $nrorg);
            $response->addDataSet(new DataSet('usuario', array('ativo' => $ativo)));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function userInfo(Request\Row $request, Response $response) {
        $userInfo = [];
        $sessionInfo = $this->session->get("LOGIN_USER_INFO");
        if(!empty($sessionInfo) && !empty($sessionInfo['USER_DATA'])) {
            $cdfilial = isset($sessionInfo['USER_DATA']['OTHER_SESSION_VALUES']['CDFILIAL']);
            $nmfilial = isset($sessionInfo['USER_DATA']['OTHER_SESSION_VALUES']['NMFILIAL']);
            $nmacesso = isset($sessionInfo['USER_DATA']['OTHER_SESSION_VALUES']['NMACESSO']);
            
            $userInfo = [
                "CDFILIALSELECIONADA" => $cdfilial == true ? $sessionInfo['USER_DATA']['OTHER_SESSION_VALUES']['CDFILIAL'] : '',
                "NMFILIAL" => $nmfilial == true ? $sessionInfo['USER_DATA']['OTHER_SESSION_VALUES']['NMFILIAL'] : '',
                "CDPICTPROD" => $sessionInfo['USER_DATA']['OTHER_SESSION_VALUES']['CDPICTPROD'],
                "CDPICTPRAT" => $sessionInfo['USER_DATA']['OTHER_SESSION_VALUES']['CDPICTPRAT'],
                "NMACESSO" => $nmacesso == true ? $sessionInfo['USER_DATA']['OTHER_SESSION_VALUES']['NMACESSO'] : ''
            ];
        }
        $response->addDataSet(new DataSet('userInfo', $userInfo));
    }
}