<?php

namespace SCM\Services;

class Inventarios {

    private $environment;
    private $entityManager;
    private $novoCodigo = array();
    
    public function __construct(\Doctrine\ORM\EntityManager $entityManager, $environment) {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
    }

    public function retornaUltimoInventario($nrorg, $cdfilial) {
        return $this->entityManager->getRepository('SCM\Model\Inventarios')->retornaUltimoInventario($nrorg, $cdfilial);
    }

    public function retornaProximoCodigoInventario($nrorg, $cdfilial) {
        return $this->entityManager->getRepository('SCM\Model\Inventarios')->retornaProximoCodigoInventario($nrorg, $cdfilial);
    }

    public function retornaProximoCodigoProdutoInventario($nrorg, $cdfilial, $nrinvent) {
        return $this->entityManager->getRepository('SCM\Model\Inventarios')->retornaProximoCodigoProdutoInventario($nrorg, $cdfilial, $nrinvent);
    }

    public function retornaDatasInventarios($nrorg, $cdfilial, $dtinicio, $dtfinal) {
        return $this->entityManager->getRepository('SCM\Model\Inventarios')->retornaDatasInventarios($nrorg, $cdfilial, $dtinicio, $dtfinal);
    }

    public function retornaProdutosInventarios($nrorg, $cdfilial, $nrinvent) {
        return $this->entityManager->getRepository('SCM\Model\Inventarios')->retornaProdutosInventarios($nrorg, $cdfilial, $nrinvent);
    }
    
    public function retornaProdutosInventariosPeriodo($nrorg, $cdfilial, $dtinicio, $dtfinal) {
        return $this->entityManager->getRepository('\SCM\Model\Inventarios')->retornaProdutosInventariosPeriodo($nrorg, $cdfilial, $dtinicio, $dtfinal);
    }

    public function criarInventario($inventarios) {

        $inventarioreti = new \SCM\Factory\Inventarioreti();
        $this->entityManager->getConnection()->beginTransaction();

        $inventariosSalvos = array();
        $nrorg = $this->environment->getNrorg();
        foreach ($inventarios as $inventario) {

            $ultimoCodigoInventario = $this->retornaCodigoInventario($inventario['CDFILIAL']);
            
            if(empty($ultimoCodigoInventario)) {
                
                $novoCodigoInventario = $this->entityManager->getRepository('SCM\Model\Inventarioreti')->codigoInventario($inventario['CDFILIAL'], $nrorg);
                $this->setNovoCodigo($novoCodigoInventario, $inventario['CDFILIAL']);

            } else {

                $novoCodigoInventario = $this->retornaNovoCodigoInventario($inventario['CDFILIAL']);
            }

            $novoInventario = $inventarioreti->criaInventario($inventario, $novoCodigoInventario, $nrorg);

            $this->entityManager->persist($novoInventario);

            $inventario['NRINVENTRETI'] = $novoCodigoInventario;
            if(!isset($inventario['PRODUTOS'])) $inventario['PRODUTOS'] = [];
            $inventario['PRODUTOS'] = $this->atualizaProdutosInventario($novoCodigoInventario, $inventario['PRODUTOS']);

            $inventariosSalvos[] = $inventario;

        }

        $this->entityManager->flush();
        $this->entityManager->getConnection()->commit();
        $this->novoCodigo = array();

        return $inventariosSalvos;


    }

    private function atualizaProdutosInventario($codigoInventario, $produtos) {
        
        for ($i = 0; $i < sizeof($produtos); $i++) {
            
            $produtos[$i]['NRINVENTRETI'] = $codigoInventario;
        }

        return $produtos;
    }

    private function retornaCodigoInventario($cdfilial) {
        if(array_key_exists($cdfilial, $this->novoCodigo)) {

            return $this->novoCodigo[$cdfilial];

        } else {

            return array();
        }
    } 

    private function setNovoCodigo($novoCodigoInventario, $cdfilial) {

        $this->novoCodigo[$cdfilial] = $novoCodigoInventario;
    }

    private function retornaNovoCodigoInventario($cdfilial) {

        $codigoAtual = intval($this->novoCodigo[$cdfilial]);
        $codigoAtual++;
        $codigoAtual = '0000000000'.$codigoAtual;
        $codigoAtual = substr($codigoAtual, -10);

        $this->novoCodigo[$cdfilial] = $codigoAtual;

        return $codigoAtual;
    }

    public function adicionarProdutos($inventariosSalvos) {

        $item = new \SCM\Factory\Iteminventreti();

        $this->entityManager->getConnection()->beginTransaction();

        foreach ($inventariosSalvos as $inventario) {
            
            foreach ($inventario['PRODUTOS'] as $produto) {
                $nrorg = $this->environment->getNrorg();
                $novoProduto = $item->adicionarProdutos($produto, $nrorg);

                $this->entityManager->persist($novoProduto);
            }
        }
        
        $this->entityManager->flush();
        $this->entityManager->getConnection()->commit();

    }
    
    public function salvarProduto($produto) {
        $item = new \SCM\Factory\Iteminventreti();
        $nrorg = $this->environment->getNrorg();
        if (!isset($produto['new'])) $produto['new'] = false;
        $produtoExists = $this->entityManager->getRepository('SCM\Model\Iteminventreti')->retornaProdutoInventario($nrorg, $produto['CDFILIAL'], $produto['NRINVENTRETI'], $produto['CDPRODUTO']);
        if (!empty($produtoExists['data']) && $produto['new']) throw new \Exception('PRODUCT_EXISTS', 500);
        else if (!empty($produtoExists['data']) && !$produto['new']) {
            $this->entityManager->getConnection()->beginTransaction();
            // $novoProduto = $item->adicionarProdutos($produto, $nrorg);
            $params = array(
                'cdfilial' => $produto['CDFILIAL'],
                'nrinventreti' => $produto['NRINVENTRETI'],
                'nritinventreti' => $produto['NRITINVENTRETI'],
                'nrorg' => $nrorg
            );
            $produtoExistente = $this->entityManager->getRepository('SCM\Model\Iteminventreti')->findOneBy($params);
            $produtoExistente->setQtitinventreti($produto['QTITINVENTRETI']);
            $this->entityManager->persist($produtoExistente);
            $this->entityManager->flush();
            $this->entityManager->getConnection()->commit();
        } else if (empty($produtoExists['data'])) {
            $this->entityManager->getConnection()->beginTransaction();
            if (!isset($produto['NRITINVENTRETI'])) {
                $codigo = $this->retornaProximoCodigoProdutoInventario($nrorg, $produto['CDFILIAL'], $produto['NRINVENTRETI']);
                if (empty($codigo)) $codigo[0]["PROXIMOCODIGO"] = "0001";
                $produto['NRITINVENTRETI'] = $codigo[0]["PROXIMOCODIGO"];
            }
            $novoProduto = $item->adicionarProdutos($produto, $nrorg);
    
            $this->entityManager->persist($novoProduto);
            $this->entityManager->flush();
            $this->entityManager->getConnection()->commit();
        } 
    }
    
    public function excluirInventario($row, $nrorg) {
        $params = array(
            'cdfilial' => $row['CDFILIAL'],
            'nrinventreti' => $row['NRINVENTRETI'],
            'nrorg' => $nrorg
        );
        
        $produtosInventario = $this->entityManager->getRepository('SCM\Model\Iteminventreti')->findBy($params);
        
        foreach($produtosInventario as $prod) {
            $this->entityManager->remove($prod);
        }
        
        $inventario = $this->entityManager->getRepository('SCM\Model\Inventarioreti')->findOneBy($params);
        
        $this->entityManager->remove($inventario);
        $this->entityManager->flush();
    }
    
    public function excluirProduto($row, $nrorg) {
        $params = array(
            'cdfilial' => $row['CDFILIAL'],
            'nrinventreti' => $row['NRINVENTRETI'],
            'nritinventreti' => $row['NRITINVENTRETI'],
            'nrorg' => $nrorg
        );

        $produto = $this->entityManager->getRepository('SCM\Model\Iteminventreti')->findOneBy($params);
        
        $this->entityManager->remove($produto);
        $this->entityManager->flush();
    }
}