import { ITekConfig } from '@zeedhi/teknisa-components-common';
import prodEnvironment from './prodEnvironment.json';
import devEnvironment from './devEnvironment.json';

export interface ScmMask extends Object {
  pattern: string;
  nivelMax?: number;
}

const env = process.env.NODE_ENV === 'production' ? prodEnvironment : devEnvironment;
const cdpictprod: string | null = localStorage.getItem('CDPICTPROD');
const displayFormat = localStorage.getItem('DATEFORMAT') || 'DD/MM/YYYY';

const decimalPlaces = '5';
const separador = localStorage.getItem('LOGIN_SEPARATOR') || '.,';
const decimalCharacter = separador.substring(2, 1) || ',';
const digitGroupSeparator = separador.substring(0, 1) || '.';

const productMask: ScmMask = {
  pattern: cdpictprod ? cdpictprod.replace(/[0-9a-zA-Z!@#$%^&*()_+]/g, 'N') : 'N.NN.NN.NNN.NN',
  nivelMax: cdpictprod ? cdpictprod.split('.').length : 5,
};

const config: ITekConfig = {
  language: 'pt-BR',
  dateFormat: 'YYYY-MM-DD',
  displayFormat,
  displayDateTimeFormat: 'DD/MM/YYYY HH:mm',
  dateTimeFormat: 'YYYY-MM-DDTHH:mm:ss',
  homeUrl: '/',
  monthFormat: 'MM/YY',
  monthDisplayFormat: 'MMM/YY',
  endPoint: env.endPoint,
  metadataEndPoint: env.metadataEndPoint,
  env: {
    product: env.product,
    baseUrl: env.baseUrl,
    modules: env.modules,
    loginRoutePath: env.modules.login.routePath,
    loginBackgroundImgUrl: env.modules.login.backgroundImgUrl,
    loginEndPoint: env.modules.login.endPoint,
  },
  saveGridLayoutsEndPoint: env.saveGridLayoutsEndPoint,
  loadGridLayoutsEndPoint: env.loadGridLayoutsEndPoint,
  masks: {
    numberMask: {
      decimalCharacter: `${decimalCharacter}`,
      digitGroupSeparator: `${digitGroupSeparator}`,
      currencySymbol: '',
      decimalPlaces: `${decimalPlaces}`,
    },
    productMask,
  },
};

export default config;
