import {
  Loader, Metadata, IEventParam, I18n, dayjs as moment, Config,
} from '@zeedhi/core';
import {
  Card, IterableComponentRender, Tabs, Tab, LoadingService, Form, Text,
} from '@zeedhi/common';
import { ScmMask } from '@/config';

export class AFController {
  private utilities = Loader.getInstance('Utilities');

  public showAfDetail = false;

  public showFilterAf = false;

  public setFilterAFs() {
    const form = Metadata.getInstance<Form>('formFilterAF');
    const dataIni = moment(form.value.PERIODO[0], 'YYYY-MM-DD').format('DD/MM/YYYY');
    const dataFin = moment(form.value.PERIODO[1], 'YYYY-MM-DD').format('DD/MM/YYYY');
    if (form.validate()) {
      const conditions = {
        CDFILIAL: localStorage.getItem('CDFILIALLOGADA') || '',
        DTINICIAL: dataIni,
        DTFINAL: dataFin,
      };
      // [
      //   {
      //     name: 'CDFILIAL',
      //     value: localStorage.getItem('CDFILIALLOGADA') || '',
      //     operator: '=',
      //   },
      //   {
      //     name: 'DATAINICIAL',
      //     value: dataIni,
      //     operator: '=',
      //   },
      //   {
      //     name: 'DATAFINAL',
      //     value: dataFin,
      //     operator: '=',
      //   },
      // ];
      const afsRender = Metadata.getInstance<IterableComponentRender>('iterableCardsAF');
      afsRender.datasource.filter = conditions;
      afsRender.datasource.get();

      this.showFilterAf = false;
    } else this.showFilterAf = true;
  }

  get isVisibleDetailList() {
    return !this.showAfDetail && !this.showFilterAf;
  }

  public setValueQtde({ component }: IEventParam<Text>) {
    const value = component.name.split('-').reverse()[0];
    // const mask = {
    //   decimalPlates: 5,
    // };
    const mask = Config.masks.productMask as ScmMask;
    const maskNumber = this.utilities.getMaskNumber(value, mask);
    component.text = `${I18n.translate('Qtde: ')} ${maskNumber}`;
  }

  public async clickCardAf({ component }: IEventParam<Card>) {
    LoadingService.show(' ');
    const mask = {
      decimalPlates: 5,
    };
    this.showAfDetail = true;
    this.showFilterAf = false;
    const tabsAfs = Metadata.getInstance<Tabs>('tabsAF');
    const afsRender = Metadata.getInstance<IterableComponentRender>('iterableCardsAF');
    tabsAfs.tabs = afsRender.datasource.data.map((row) => new Tab({
      name: `${row.CDPRODUTO}`,
      tabTitle: `${this.utilities.getValueWithMaskProduto(row.CDPRODUTO)} - ${row.NMPRODUTO}`,
      children: [
        {
          allowDuplicate: true,
          name: `tagTab-${row.CDPRODUTO}`,
          component: 'ZdTag',
          tag: 'div',
          isVisible: false,
        },
        {
          name: `formDetailAf-${row.CDPRODUTO}`,
          component: 'ZdForm',
          children: [
            {
              allowDuplicate: true,
              name: 'CDPRODUTO',
              component: 'ZdText',
              readonly: true,
              text: `${I18n.translate('Produto: ')} ${this.utilities.getValueWithMaskProduto(row.CDPRODUTO)} ${' | '}
              ${row.NMPRODUTO}`,
              grid: {
                cols: 12,
              },
            },
            {
              allowDuplicate: true,
              name: 'SGUNIDADE',
              component: 'ZdText',
              readonly: true,
              text: `${I18n.translate('Unidade: ')} ${row.SGUNIDADE}`,
              grid: {
                cols: 12,
              },
            },
            {
              allowDuplicate: true,
              name: 'QTPROGENTAF',
              component: 'ZdText',
              readonly: true,
              text: `${I18n.translate('Qtde: ')} ${this.utilities.getMaskNumber(row.QTPROGENTAF, mask)}`,
              grid: {
                cols: 12,
              },
            },
            {
              allowDuplicate: true,
              name: 'NMMARCA',
              component: 'ZdText',
              readonly: true,
              text: `${I18n.translate('Marca: ')} ${row.NMMARCA}`,
              grid: {
                cols: 12,
              },
            },
            {
              allowDuplicate: true,
              name: 'DTPROGENTAF',
              component: 'ZdText',
              readonly: true,
              text: `${I18n.translate('Dt Entrega: ')} ${row.DTPROGENTAF}`,
              grid: {
                cols: 12,
              },
            },
            {
              allowDuplicate: true,
              name: 'VRPREUNIAF',
              component: 'ZdText',
              readonly: true,
              text: `${I18n.translate('Valor Unitário: ')} ${this.utilities.getMaskNumber(row.VRPREUNIAF, mask)}`,
              grid: {
                cols: 12,
              },
            },
            {
              allowDuplicate: true,
              name: 'VRTOTALITEM',
              component: 'ZdText',
              readonly: true,
              text: `${I18n.translate('Valor Total: ')} ${this.utilities.getMaskNumber(row.VRTOTALITEM, mask)}`,
              grid: {
                cols: 12,
              },
            },
            {
              allowDuplicate: true,
              name: 'NRAUTOFORN',
              component: 'ZdText',
              readonly: true,
              text: `${I18n.translate('AF: ')} ${row.NRAUTOFORN}`,
              grid: {
                cols: 12,
              },
            },
            {
              allowDuplicate: true,
              name: 'CDFILLANC',
              component: 'ZdText',
              readonly: true,
              text: `${I18n.translate('Unidade Lançamento: ')} ${row.CDFILLANC}`,
              grid: {
                cols: 12,
              },
            },
            {
              allowDuplicate: true,
              name: 'CDFILDEST',
              component: 'ZdText',
              readonly: true,
              text: `${I18n.translate('Unidade Destino: ')} ${row.CDFILDEST}`,
              grid: {
                cols: 12,
              },
            },
          ],
        },
      ],
    }));
    const index = component.name;
    tabsAfs.activeTab = await tabsAfs.tabs.findIndex((tab) => tab.children[0].name === `tagTab-${index}`);
    LoadingService.hide();
  }

  public setFilterMountedAfs() {
    this.showFilterAf = true;
  }

  public clickBackAFs() {
    this.showAfDetail = false;
    // this.showInventarioDetail = true;
    const afs = Metadata.getInstance<IterableComponentRender>('iterableCardsAF');
    afs.reload();
  }

  public orderAf() {
    const afs = Metadata.getInstance<IterableComponentRender>('iterableCardsAF');
    afs.datasource.clearOrder();
    afs.datasource.addOrder('NRAUTOFORN', 'asc');
  }

  public orderProduto() {
    const afs = Metadata.getInstance<IterableComponentRender>('iterableCardsAF');
    afs.datasource.clearOrder();
    afs.datasource.addOrder('CDPRODUTO', 'asc');
  }

  public orderDtEntrega() {
    const afs = Metadata.getInstance<IterableComponentRender>('iterableCardsAF');
    afs.datasource.clearOrder();
    afs.datasource.addOrder('DTPROGENTAF', 'asc');
  }
}
