import {
  IEventParam, Metadata, IDictionary, I18n, Loader, DatasourceFactory,
} from '@zeedhi/core';
import {
  Card, IterableComponentRender, Tabs, Tab, LoadingService, Increment, Button, Form,
  Text, DialogService,
} from '@zeedhi/common';
import { BottomSheet } from '@zeedhi/zd-bottom-sheet';
import { HttpCRUDTranslator } from '@/http-crud-translator/http-crud-translator';

export class EfetivosController {
  private showEditServicoEfetivos = false;

  private showClientesEfetivos = false;

  private utilities = Loader.getInstance('Utilities');

  private serviceSelected = {
    CDSERVFILI: '',
    CDTIPOCARD: '',
    DTCARDIAPME: '',
    TOTAL: 0,
  };

  private periodo = {
    DTINICIAL: '',
    DTFINAL: '',
  };

  private editingClientName = '';

  get isVisibleEfetivosList() {
    return !this.showEditServicoEfetivos && !this.showClientesEfetivos;
  }

  public get setFilterEfetivos() {
    return {
      CDFILIAL: localStorage.getItem('CDFILIALLOGADA') || '',
      DTINICIAL: this.periodo.DTINICIAL || '',
      DTFINAL: this.periodo.DTFINAL || '',
    };
    // [
    //   {
    //     name: 'CDFILIAL',
    //     value: localStorage.getItem('CDFILIALLOGADA') || '',
    //     operator: '=',
    //   },
    //   {
    //     name: 'DTINICIAL',
    //     value: this.periodo.DTINICIAL || '',
    //     operator: '=',
    //   },
    //   {
    //     name: 'DTFINAL',
    //     value: this.periodo.DTFINAL || '',
    //     operator: '=',
    //   },
    // ];
  }

  public clickBackEditServico() {
    this.showEditServicoEfetivos = false;
    const servicos = Metadata.getInstance<IterableComponentRender>('iterableCardsEfetivos');
    servicos.reload();
  }

  public async clickCardServicoEfetivos({ component }: IEventParam<Card>) {
    LoadingService.show(' ');
    this.showEditServicoEfetivos = true;
    const tabsServicos = Metadata.getInstance<Tabs>('tabsServicos');
    const servicosRender = Metadata.getInstance<IterableComponentRender>('iterableCardsEfetivos');
    if (tabsServicos.tabs.length > 0) {
      await Promise.all(tabsServicos.tabs.map(async (tab: IDictionary) => {
        await tab.children.map(async (child: IDictionary) => {
          await tab.removeChild(child.name);
        });
      })).then(() => {
        tabsServicos.tabs = [];
        tabsServicos.tabs = servicosRender.datasource.data.map((row) => this.newTabs(row));
      });
    } else tabsServicos.tabs = servicosRender.datasource.data.map((row) => this.newTabs(row));
    const index = component.name.split('-')[1];
    tabsServicos.activeTab = await tabsServicos.tabs.findIndex((tab) => tab.children[0].name === `tagTab-${index}`);
    LoadingService.hide();
  }

  private newTabs(row: IDictionary) {
    return new Tab({
      name: `${row.CDSERVFILI}`,
      tabTitle: `${row.CDSERVFILI} - ${row.NMSERVICO}`,
      children: [
        {
          allowDuplicate: true,
          name: `tagTab-${row.CDSERVFILI}_${row.NRCARDPME}_${row.NRCARDIAPME}`,
          component: 'ZdTag',
          tag: 'div',
          children: [
            {
              name: `formEditService-${row.CDSERVFILI}_${row.DTCARDIAPME}_${row.NRCARDIAPME}`,
              component: 'ZdForm',
              children: [
                {
                  allowDuplicate: true,
                  name: 'DATA_EXTENSO',
                  component: 'ZdText',
                  // label: 'Data',
                  // value: `${row.DATA_EXTENSO}`,
                  text: `${I18n.translate('Data: ')} ${row.DATA_EXTENSO}`,
                  grid: {
                    cols: 12,
                  },
                },
                {
                  allowDuplicate: true,
                  isVisible: false,
                  name: 'CDTIPOCARD',
                  component: 'ZdTextInput',
                  readonly: true,
                  label: 'Código Tipo de Cardápio',
                  value: `${row.CDTIPOCARD}`,
                  grid: {
                    cols: 12,
                  },
                },
                {
                  allowDuplicate: true,
                  name: 'TIPOCARD',
                  component: 'ZdText',
                  // label: 'Tipo de Cardápio',
                  // value: `${row.CDTIPOCARD} - ${row.NMTIPOCARD}`,
                  text: `${I18n.translate('Tipo de Cardápio: ')} ${row.CDTIPOCARD} - ${row.NMTIPOCARD}`,
                  grid: {
                    cols: 12,
                  },
                },
                {
                  allowDuplicate: true,
                  name: 'DSCARDPME',
                  component: 'ZdText',
                  // label: 'Cardápio',
                  // value: `${row.DSCARDPME}`,
                  text: `${I18n.translate('Cardápio: ')} ${row.DSCARDPME}`,
                  grid: {
                    cols: 12,
                  },
                },
                {
                  allowDuplicate: true,
                  name: 'QTCLIEFECPME',
                  component: 'ZdIncrement',
                  label: 'Clientes',
                  clearable: false,
                  step: 1,
                  events: {
                    change: '{{EfetivosController.changeEfetivosServico}}',
                  },
                  validations: {
                    required: {},
                    min: {
                      limit: 0,
                    },
                  },
                  mask: {
                    decimalPlaces: 0,
                  },
                  value: row.QTCLIEFECPME,
                  grid: {
                    cols: 12,
                  },
                },
                {
                  allowDuplicate: true,
                  name: 'QTFUNEFECPME',
                  component: 'ZdIncrement',
                  label: 'Funcionários',
                  clearable: false,
                  step: 1,
                  events: {
                    change: '{{EfetivosController.changeEfetivosServico}}',
                  },
                  validations: {
                    required: {},
                    min: {
                      limit: 0,
                    },
                  },
                  mask: {
                    decimalPlaces: 0,
                  },
                  value: row.QTFUNEFECPME,
                  grid: {
                    cols: 12,
                  },
                },
                {
                  allowDuplicate: true,
                  name: 'QTDOAEFECPME',
                  component: 'ZdIncrement',
                  label: 'Doação',
                  clearable: false,
                  step: 1,
                  events: {
                    change: '{{EfetivosController.changeEfetivosServico}}',
                  },
                  validations: {
                    required: {},
                    min: {
                      limit: 0,
                    },
                  },
                  mask: {
                    decimalPlaces: 0,
                  },
                  value: row.QTDOAEFECPME,
                  grid: {
                    cols: 12,
                  },
                },
                {
                  allowDuplicate: true,
                  name: 'QTCOMEFECPME',
                  component: 'ZdNumber',
                  readonly: true,
                  showBorder: false,
                  label: 'Total de Efetivos',
                  value: row.QTCOMEFECPME,
                  cssClass: 'readonly-text',
                  mask: {
                    decimalPlaces: 0,
                  },
                  grid: {
                    cols: 12,
                  },
                },
                {
                  allowDuplicate: true,
                  name: 'CDFILCENTPME',
                  isVisible: false,
                  component: 'ZdTextInput',
                  value: `${row.CDFILCENTPME}`,
                },
                {
                  allowDuplicate: true,
                  name: 'NRCARDPME',
                  isVisible: false,
                  component: 'ZdTextInput',
                  value: `${row.NRCARDPME}`,
                },
                {
                  allowDuplicate: true,
                  name: 'NRCARDIAPME',
                  isVisible: false,
                  component: 'ZdTextInput',
                  value: `${row.NRCARDIAPME}`,
                },
                {
                  allowDuplicate: true,
                  name: 'DTCARDIAPME',
                  isVisible: false,
                  component: 'ZdTextInput',
                  value: `${row.DTCARDIAPME}`,
                },
                {
                  allowDuplicate: true,
                  name: 'CDSERVFILI',
                  isVisible: false,
                  component: 'ZdTextInput',
                  value: `${row.CDSERVFILI}`,
                },
              ],
            },
            {
              allowDuplicate: true,
              name: 'clientesList',
              component: 'ZdList',
              divided: true,
              dense: true,
              twoLine: true,
              items: [
                {
                  allowDuplicate: true,
                  name: 'clientesItem',
                  component: 'ZdListItem',
                  title: 'Clientes',
                  subtitle: 'Ajustar quantidades',
                  rightField: {
                    allowDuplicate: true,
                    name: 'rightIcon1',
                    component: 'ZdButton',
                    iconName: 'mdi-tune',
                    icon: true,
                    flat: true,
                    events: {
                      click: '{{EfetivosController.openClientesEfetivos}}',
                    },
                  },
                },
              ],
            },
            {
              name: `salvarEditServicoBtn-${row.CDSERVFILI}_${row.DTCARDIAPME}_${row.NRCARDIAPME}`,
              component: 'ZdButton',
              label: 'Salvar',
              block: true,
              fixed: true,
              bottom: true,
              disabled: true,
              events: {
                click: '{{EfetivosController.salvarEditServico}}',
              },
              cssStyle: {
                right: '0px',
              },
            },
          ],
        },
      ],
    });
  }

  public salvarEditServico({ component }: IEventParam<Button>) {
    const form = component.parent?.getChildInstance<Form>(component.parent?.children[0].name);
    if (form?.validate()) {
      const dataset = [
        {
          CDFILIAL: localStorage.getItem('CDFILIALLOGADA') || '',
          ...form?.value,
        },
      ];
      LoadingService.show(' ');
      HttpCRUDTranslator.post('/servicosCardapioEfetivos/update', { requestType: 'DataSet', dataset })
        .then(async (response: IDictionary) => {
          if (!response.data.error) {
            this.serviceSelected = {
              CDSERVFILI: form?.value.CDSERVFILI,
              CDTIPOCARD: form?.value.CDTIPOCARD,
              DTCARDIAPME: form?.value.DTCARDIAPME,
              TOTAL: form?.value.QTCOMEFECPME,
            };
            const tabsServicos = Metadata.getInstance<Tabs>('tabsServicos');
            const clientesRender = Metadata.getInstance<IterableComponentRender>('iterableCardsClientesEfetivos');
            const cloneConfig = clientesRender.datasource.clone();
            const cloneDs = DatasourceFactory.factory(cloneConfig);
            await cloneDs.get().then(async (clientes: IDictionary[]) => {
              if (clientes.length > 0) {
                clientes.forEach((cliente: IDictionary) => {
                  cliente.QTCOMEFCLPME = 0;
                  cliente.QTDOAEFCLPME = 0;
                  cliente.QTFUNEFCLPME = 0;
                  cliente.TOTAL = 0;
                  cliente.changed = true;
                });
                const clientePrincipal = clientes.filter((cliente: IDictionary) => cliente.IDCLIENPRIN === 'S');
                if (clientePrincipal.length > 0) {
                  const index = clientes.indexOf(clientePrincipal[0]);
                  clientePrincipal[0].QTCOMEFCLPME = form?.value.QTCLIEFECPME;
                  clientePrincipal[0].QTDOAEFCLPME = form?.value.QTDOAEFECPME;
                  clientePrincipal[0].QTFUNEFCLPME = form?.value.QTFUNEFECPME;
                  clientePrincipal[0].TOTAL = form?.value.QTCOMEFECPME;
                  if (index !== -1) {
                    const [aux] = clientePrincipal;
                    clientes[index] = aux;
                  }
                } else {
                  clientes[0].QTCOMEFCLPME = form?.value.QTCLIEFECPME;
                  clientes[0].QTDOAEFCLPME = form?.value.QTDOAEFECPME;
                  clientes[0].QTFUNEFCLPME = form?.value.QTFUNEFECPME;
                  clientes[0].TOTAL = form?.value.QTCOMEFECPME;
                }
                await HttpCRUDTranslator.post('/clientesDoServicoEfetivos/update',
                  { requestType: 'DataSet', dataset: [...clientes] })
                  .then(() => {
                    if (!response.data.error) {
                      // this.utilities.showAlert('successSave', 'SUCCESS_SAVE', 'green');
                      this.utilities.showAlert('alertQuantClientes', 'SETUP_CLIENT_QUANT', 'warning');
                      component.disabled = true;
                      tabsServicos.nextTab();
                    } else {
                      const mensagem = response.data.error;
                      this.utilities.showAlert('errorSave', I18n.translate(mensagem), 'error');
                    }
                  }).catch((error: string) => {
                    this.utilities.showAlert('alert', error, 'error');
                  });
              } else {
                this.utilities.showAlert('successSave', 'SUCCESS_SAVE', 'green');
                component.disabled = true;
                tabsServicos.nextTab();
              }
            });
          } else {
            const mensagem = response.data.error;
            this.utilities.showAlert('errorSave', I18n.translate(mensagem), 'error');
          }
        }).catch((error: string) => {
          this.utilities.showAlert('alert', error, 'error');
        }).finally(() => {
          LoadingService.hide();
        });
    }
  }

  public changeEfetivosServico({ component }: IEventParam<Increment>) {
    const form = component.parent as Form;
    if (form.validate()) {
      form.value.QTCOMEFECPME = form.value.QTCLIEFECPME + form.value.QTFUNEFECPME + form.value.QTDOAEFECPME;
      const size = component.parent?.parent?.children?.length || 0;
      const btn = component.parent?.parent?.children[size - 1] as Button;
      btn.disabled = false;
    }
  }

  public openClientesEfetivos() {
    this.showEditServicoEfetivos = false;
    this.showClientesEfetivos = true;
    const tabsServicos = Metadata.getInstance<Tabs>('tabsServicos');
    const selectedTab = tabsServicos.tabs[tabsServicos.activeTab] as Tab;
    // const children = selectedTab?.childrenInstances[1]?.childrenInstances;
    const tag = selectedTab?.childrenInstances.find((data: IDictionary) => data.component === 'ZdTag');
    const formName = tag?.childrenInstances.find((data: IDictionary) => data.component === 'ZdForm');
    const form = formName as Form;
    this.serviceSelected = {
      CDSERVFILI: form.value.CDSERVFILI,
      CDTIPOCARD: form.value.CDTIPOCARD,
      DTCARDIAPME: form.value.DTCARDIAPME,
      TOTAL: form.value.QTCOMEFECPME,
    };
    const clientesRender = Metadata.getInstance<IterableComponentRender>('iterableCardsClientesEfetivos');
    clientesRender.reload();
  }

  public clickBackClientesEfetivos() {
    this.showClientesEfetivos = false;
    this.showEditServicoEfetivos = true;
  }

  public get setFilterClientesEfetivos() {
    return {
      CDFILIAL: localStorage.getItem('CDFILIALLOGADA') || '',
      DTINICIAL: this.serviceSelected.DTCARDIAPME || '',
      DTFINAL: this.serviceSelected.DTCARDIAPME || '',
      PERIODO: 5,
      CDSERVFILI: this.serviceSelected.CDSERVFILI || '',
      CDTIPOCARD: this.serviceSelected.CDTIPOCARD || '',
    };
    // [
    //   {
    //     name: 'CDFILIAL',
    //     value: localStorage.getItem('CDFILIALLOGADA') || '',
    //     operator: '=',
    //   },
    //   {
    //     name: 'DTINICIAL',
    //     value: this.serviceSelected.DTCARDIAPME || '',
    //     operator: '=',
    //   },
    //   {
    //     name: 'DTFINAL',
    //     value: this.serviceSelected.DTCARDIAPME || '',
    //     operator: '=',
    //   },
    //   {
    //     name: 'PERIODO',
    //     value: 5,
    //     operator: '=',
    //   },
    //   {
    //     name: 'CDSERVFILI',
    //     value: this.serviceSelected.CDSERVFILI || '',
    //     operator: '=',
    //   },
    //   {
    //     name: 'CDTIPOCARD',
    //     value: this.serviceSelected.CDTIPOCARD || '',
    //     operator: '=',
    //   },
    // ];
  }

  public clickFilterEfetivos() {
    const bottomSheet = Metadata.getInstance<BottomSheet>('filterSheet');
    bottomSheet.isVisible = true;
    bottomSheet.isOpen = true;
  }

  public clickCloseFilterBtn() {
    const bottomSheet = Metadata.getInstance<BottomSheet>('filterSheet');
    bottomSheet.isVisible = false;
    bottomSheet.isOpen = false;
  }

  public setPeriodoEfetivos({ component }: IEventParam<Button>) {
    const form = component.parent as Form;
    if (form.validate()) {
      [this.periodo.DTINICIAL, this.periodo.DTFINAL] = form.value.PERIODO;
      const efetivos = Metadata.getInstance<IterableComponentRender>('iterableCardsEfetivos');
      efetivos.datasource.setSearch(form.value.search);
      efetivos.datasource.clearOrder();
      efetivos.reload();
      const bottomSheet = Metadata.getInstance<BottomSheet>('filterSheet');
      bottomSheet.isVisible = false;
      bottomSheet.isOpen = false;
    }
  }

  public openFilter() {
    const btn = Metadata.getInstance<Button>('filterEfetivosButton');
    btn.click();
  }

  public async clickCardCliente({ component }: IEventParam<Card>) {
    const bottomSheet = Metadata.getInstance<BottomSheet>('editClientSheet');
    const [cdcliente, cdservfili] = component.name.split('-')[1].split('_');
    const clientsCards = Metadata.getInstance<IterableComponentRender>('iterableCardsClientesEfetivos');
    const clientSelected = await clientsCards.datasource.data.find(
      (data: IDictionary) => data.CDCLIENTE === cdcliente && data.CDSERVFILI === cdservfili,
    );
    this.editingClientName = clientSelected?.NMRAZSOCCLIE;
    const form = Metadata.getInstance<Form>('formEditClient');
    form.value.TOTAL = clientSelected?.TOTAL || 0;
    form.value.QTCOMEFCLPME = clientSelected?.QTCOMEFCLPME || 0;
    form.value.QTDOAEFCLPME = clientSelected?.QTDOAEFCLPME || 0;
    form.value.QTFUNEFCLPME = clientSelected?.QTFUNEFCLPME || 0;
    form.value.CDCLIENTE = clientSelected?.CDCLIENTE;
    form.value.CDFILCENTPME = clientSelected?.CDFILCENTPME;
    form.value.DTEFETCLPME = clientSelected?.DTEFETCLPME;
    bottomSheet.isVisible = true;
    bottomSheet.isOpen = true;
  }

  public clickCloseEditClient() {
    const bottomSheet = Metadata.getInstance<BottomSheet>('editClientSheet');
    bottomSheet.isVisible = false;
    bottomSheet.isOpen = false;
  }

  public ordenaServicosEfetivos({ component }: IEventParam<Button>) {
    const { name } = component;
    const efetivos = Metadata.getInstance<IterableComponentRender>('iterableCardsEfetivos');
    const ds = efetivos.datasource;
    const order = ds.getOrderByColumn(name);
    if (order) {
      if (order === 'desc') ds.addOrder(name, 'asc');
      else ds.addOrder(name, 'desc');
    } else ds.addOrder(name, 'desc');
  }

  public setValueEfetivos({ component }: IEventParam<Text>) {
    let value = component.name.split('-')[1].split('_')[2];
    // if (value !== 'null') {
    if (value === 'null') value = '0';
    component.text = `Total de Efetivos: ${value}`;
    // }
  }

  public salvarEditClient({ component }: IEventParam<Button>) {
    const form = component.parent as Form;
    if (form.validate()) {
      const clientes = Metadata.getInstance<IterableComponentRender>('iterableCardsClientesEfetivos');
      const cliente = clientes.datasource.data.find(
        (data) => data.CDSERVFILI === this.serviceSelected.CDSERVFILI && data.CDCLIENTE === form.value.CDCLIENTE,
      );
      if (cliente) {
        cliente.TOTAL = form.value.TOTAL;
        cliente.QTCOMEFCLPME = form.value.QTCOMEFCLPME;
        cliente.QTDOAEFCLPME = form.value.QTDOAEFCLPME;
        cliente.QTFUNEFCLPME = form.value.QTFUNEFCLPME;
      }
      const bottomSheet = Metadata.getInstance<BottomSheet>('editClientSheet');
      bottomSheet.isVisible = false;
      bottomSheet.isOpen = false;
    }
  }

  public changeEfetivosClient({ component }: IEventParam<Increment>) {
    const form = component.parent as Form;
    if (form.validate()) form.value.TOTAL = form.value.QTCOMEFCLPME + form.value.QTFUNEFCLPME + form.value.QTDOAEFCLPME;
  }

  public async salvarClients() {
    const clientes = Metadata.getInstance<IterableComponentRender>('iterableCardsClientesEfetivos');
    if (clientes.datasource.data.length > 0) {
      let total = 0;
      await clientes.datasource.data.forEach((row) => {
        total += row.TOTAL;
      });
      if (this.serviceSelected.TOTAL === total) {
        const dataset = [...clientes.datasource.data];
        LoadingService.show(' ');
        HttpCRUDTranslator.post('/clientesDoServicoEfetivos/update', { requestType: 'DataSet', dataset })
          .then((response: IDictionary) => {
            if (!response.data.error) {
              this.utilities.showAlert('successSave', 'SUCCESS_SAVE', 'green');
              this.clickBackClientesEfetivos();
            } else {
              const mensagem = response.data.error;
              this.utilities.showAlert('errorSave', I18n.translate(mensagem), 'error');
            }
          }).catch((error: string) => {
            this.utilities.showAlert('alert', error, 'error');
          }).finally(() => {
            LoadingService.hide();
          });
      } else {
        DialogService.show({
          name: 'erroSomaClientes',
          type: 'error',
          title: I18n.translate('scm::ATTENTION'),
          text: I18n.translate('scm::STAFF_CLIENT_SUM_NOT_EQUAL'),
          buttons: [
            {
              allowDuplicate: true,
              name: 'ok',
              component: 'ZdButton',
              label: I18n.translate('scm::OK'),
              events: { click: () => DialogService.hide() },
            },
          ],
        });
      }
    } else {
      this.clickBackClientesEfetivos();
    }
  }
}
