import {
  Loader, Metadata, IDictionary, I18n, Http, Router,
} from '@zeedhi/core';
import { Form, DialogService, LoadingService } from '@zeedhi/common';
import { callMakeServer } from '../server/server';
import { getConfig } from '../server/config';

export class ImportacaoController {
  private utilities = Loader.getInstance('Utilities');

  public unidades: IDictionary = this.utilities.getItemLocalStorage('Units') || [];

  public agrupamentosProdutos: IDictionary = this.utilities.getItemLocalStorage('GruposProduto') || [];

  public async importar() {
    const form = Metadata.getInstance<Form>('formImportacao');
    if (form.validate()) {
      const { value } = form;
      if (this.utilities.validateInterval(value.INTERVAL[0], value.INTERVAL[1])) {
        if (localStorage.getItem('hasImportedData') !== 'true') {
          this.continueImportacao();
        } else this.hasImportedData();
      }
    }
  }

  public continueImportacao() {
    this.validateActiveUser().then((valid) => {
      if (valid) {
        LoadingService.show(' ');
        const form = Metadata.getInstance<Form>('formImportacao');
        const { value } = form;
        localStorage.setItem('FILIAIS_IMPORTACAO', value.UNIDADE.map((u: any) => u.CDFILIAL).join('_'));
        localStorage.setItem('DATA_INICIAL', value.INTERVAL[0]);
        localStorage.setItem('DATA_FINAL', value.INTERVAL[1]);
        const cdgrupprod = value.AGRUPAMENTO_PRODUTO ? value.AGRUPAMENTO_PRODUTO : 'T';
        localStorage.setItem('CDGRUPPROD', cdgrupprod);
        const serverConfig = getConfig();
        callMakeServer(serverConfig).then(() => {
          setTimeout(() => {
            LoadingService.hide();
          }, 5000);
          localStorage.setItem('hasImportedData', 'true');
          DialogService.show({
            name: 'successImport',
            type: 'success',
            title: I18n.translate('scm::SUCCESS'),
            text: I18n.translate('scm::SUCCESS_IMPORT'),
            buttons: [
              {
                name: 'ok',
                component: 'ZdButton',
                label: I18n.translate('scm::OK'),
                events: {
                  click: () => {
                    DialogService.hide();
                    Router.push('/');
                  },
                },
              },
            ],
          });
        });
      }
    });
  }

  public validateActiveUser() {
    const filter = [
      {
        name: 'CDOPERADOR',
        operator: '=',
        value: localStorage.getItem('CDOPERADOR'),
      },
    ];
    LoadingService.show(' ');
    return Http.post('/OperadorAtivo', { requestType: 'FilterData', filter }).then((result: IDictionary) => {
      if (!result) {
        this.utilities.showAlert('inactiveUser', 'INACTIVE_USER', 'error');
        // logout
        return false;
      }
      return true;
    }).finally(() => {
      LoadingService.hide();
    });
  }

  public hasImportedData() {
    // this.utilities.showAlert('alreadyImported', 'HAS_IMPORTED_DATA', 'e')
    DialogService.show({
      name: 'confirmImport',
      type: 'warning',
      title: I18n.translate('scm::ATTENTION'),
      text: I18n.translate('scm::HAS_IMPORTED_DATA'),
      buttons: [
        {
          name: 'yes',
          component: 'ZdButton',
          label: I18n.translate('scm::YES'),
          events: { click: () => this.continueImportacao() },
          outline: true,
        },
        {
          name: 'no',
          component: 'ZdButton',
          label: I18n.translate('scm::NO'),
          events: { click: () => DialogService.hide() },
        },
      ],
    });
  }
}
