import {
  Config, Singleton, Http, IDictionary, Loader, dayjs as moment, I18n,
} from '@zeedhi/core';
import { ScmMask } from '@/config';
import 'dayjs/locale/pt';
import 'dayjs/locale/en';
import 'dayjs/locale/es';

@Singleton
export class LoginController {
  private importacaoController = Loader.getInstance('ImportacaoController');

  private utilities = Loader.getInstance('Utilities');

  public appController = Loader.getInstance('AppController');

  get logo() {
    return Config.env.product.logoUrl;
  }

  get backgroundImgUrl() {
    const url = Config.env.loginBackgroundImgUrl;
    return `url(${url})`;
  }

  get endPoint() {
    return Config.env.loginEndPoint;
  }

  public async setLocalStorageUser(userData: IDictionary) {
    // const query = filters.reduce((result, filter) =>
    // (`${result}${filter.name}${filter.operator}${filter.value}&`), '?').slice(0, -1);
    const loadData = [
      // this.utilities.startIndexedDB(),
      // Http.get(`/Filial${query}`),
      // Http.post('/GruposProduto'),
    ];
    if (userData) loadData.push(Http.post('/userInfo', { requestType: 'Row', row: {} }));
    await Promise.all(loadData).then((result: IDictionary) => {
      if (userData) {
        const userInfo = result[0].data?.dataset?.userInfo;
        if (userInfo) {
          localStorage.setItem('CDFILIALLOGADA', userInfo.CDFILIALSELECIONADA);
          localStorage.setItem('NMFILIALLOGADA', userInfo.NMFILIAL);
          localStorage.setItem('NMACESSO', userInfo.NMACESSO);
          localStorage.setItem('CDPICTPROD', userInfo.CDPICTPROD);
          localStorage.setItem('CDPICTPRAT', userInfo.CDPICTPRAT);
          (Config.masks.productMask as ScmMask).pattern = userInfo.CDPICTPROD
            ? userInfo.CDPICTPROD.replace(/[0-9a-zA-Z!@#$%^&*()_+]/g, 'N')
            : 'N.NN.NN.NNN.NN';

          (Config.masks.productMask as ScmMask).nivelMax = userInfo.CDPICTPROD
            ? userInfo.CDPICTPROD.split('.').length
            : 5;
        }
      }
    });

    this.setLanguage();

    localStorage.setItem('DATEFORMAT', this.utilities.getFormatLocale());
    Config.displayFormat = this.utilities.getFormatLocale();
  }

  private setLanguage() {
    const lang = this.utilities.getLanguage();
    I18n.changeLanguage(this.appController.metaLanguages[lang]);
    moment.locale(lang.substring(0, 2));
  }
}
