import { Config } from '@zeedhi/core';
import { IExternalColumn, IRouteConfig } from '@zeedhi/zeedhi-miragejs';
import qs from 'qs';

const getFilial = (): IRouteConfig<IExternalColumn> => {
  const filter = {
    CDOPERADOR: localStorage.getItem('CDOPERADOR') || '',
  };
  const filial: IRouteConfig<IExternalColumn> = {
    externalApi: true,
    persist: true,
    apiUrl: `${Config.endPoint}/filial?${qs.stringify(filter)}`,
    dataColumn: 'data',
    uniqueKey: 'CDFILIAL',
    columns: {
      // id: {},
      CDFILIAL: {},
      CDEMPRESA: {},
      CDFILCENTCMP: {},
      CDFILCENTPME: {},
      CDFILLANCA: {},
      DESCRICAO: {},
      NMFILIAL: {},
    },
  };
  return filial;
};

const getInventarios = (): IRouteConfig<IExternalColumn> => {
  const filter = {
    CDFILIAL: localStorage.getItem('FILIAIS_IMPORTACAO') || '',
    DTINICIAL: localStorage.getItem('DATA_INICIAL') || '',
    DTFINAL: localStorage.getItem('DATA_FINAL') || '',
  };
  const inventarios: IRouteConfig<IExternalColumn> = {
    externalApi: true,
    persist: true,
    apiUrl: `${Config.endPoint}/inventarios?${qs.stringify(filter)}`,
    dataColumn: 'data',
    uniqueKey: 'NRINVENTRETI',
    columns: {
      // id: {},
      CDFILIAL: {},
      DSINVENTRETI: {},
      DTINVENTRETI: {},
      NRINVENTRETI: {},
    },
  };
  return inventarios;
};

const getProdutos = (): IRouteConfig<IExternalColumn> => {
  const filter = {
    CDFILIAL: localStorage.getItem('FILIAIS_IMPORTACAO') || '',
    CDGRUPPROD: localStorage.getItem('CDGRUPPROD') || '',
  };
  const produtos: IRouteConfig<IExternalColumn> = {
    externalApi: true,
    persist: true,
    apiUrl: `${Config.endPoint}/produtos?${qs.stringify(filter)}`,
    dataColumn: 'data',
    uniqueKey: 'id',
    columns: {
      id: {},
      CDARVPROD: {},
      CDFILIAL: {},
      CDPRODUTO: {},
      NMPRODUTO: {},
      SGUNIDADE: {},
      TIPOPRODUTO: {},
    },
  };
  return produtos;
};

const getProdutosInventario = (): IRouteConfig<IExternalColumn> => {
  const filter = {
    CDFILIAL: localStorage.getItem('FILIAIS_IMPORTACAO') || '',
    NRINVENTRETI: null,
    DTINICIAL: localStorage.getItem('DATA_INICIAL') || '',
    DTFINAL: localStorage.getItem('DATA_FINAL') || '',
  };
  const produtosInventario: IRouteConfig<IExternalColumn> = {
    externalApi: true,
    persist: true,
    apiUrl: `${Config.endPoint}/produtosInventario?${qs.stringify(filter)}`,
    dataColumn: 'data',
    uniqueKey: 'CDPRODUTO',
    columns: {
      // id: {},
      CDARVPROD: {},
      CDFILIAL: {},
      CDOPERINCLUSAO: {},
      CDOPERULTATU: {},
      CDPRODUTO: {},
      DTINCLUSAO: {},
      DTULTATU: {},
      NMPRODUTO: {},
      NRINVENTRETI: {},
      NRITINVENTRETI: {},
      QTITINVENTRETI: {},
    },
  };
  return produtosInventario;
};

const getAF = (): IRouteConfig<IExternalColumn> => {
  const filter = {
    CDFILIAL: localStorage.getItem('FILIAIS_IMPORTACAO') || '',
    DTINICIAL: localStorage.getItem('DATA_INICIAL') || '',
    DTFINAL: localStorage.getItem('DATA_FINAL') || '',
  };
  const af: IRouteConfig<IExternalColumn> = {
    externalApi: true,
    persist: true,
    apiUrl: `${Config.endPoint}/af?${qs.stringify(filter)}`,
    dataColumn: 'data',
    uniqueKey: 'id',
    columns: {
      id: {},
      NRORG: {},
      CDFILDEST: {},
      CDFILLANC: {},
      NRAUTOFORN: {},
      CDPRODUTO: {},
      NMPRODUTO: {},
      SGUNIDADE: {},
      CDMARCA: {},
      NMMARCA: {},
      DTPROGENTAF: {},
      VRPERDESAF: {},
      QTPROGENTAF: {},
      VRPREUNIAF: {},
      VRDESCITMAF: {},
      VRTOTALITEM: {},
    },
  };
  return af;
};

const getClientesDoServicoEfetivos = (): IRouteConfig<IExternalColumn> => {
  const filter = {
    CDFILIAL: localStorage.getItem('FILIAIS_IMPORTACAO') || '',
    DTINICIAL: localStorage.getItem('DATA_INICIAL') || '',
    DTFINAL: localStorage.getItem('DATA_FINAL') || '',
    PERIODO: 5,
    CDSERVFILI: 'T',
    CDTIPOCARD: 'T',
  };
  const clientesDoServicoEfetivos: IRouteConfig<IExternalColumn> = {
    externalApi: true,
    persist: true,
    apiUrl: `${Config.endPoint}/clientesDoServicoEfetivos?${qs.stringify(filter)}`,
    dataColumn: 'data',
    uniqueKey: 'id',
    columns: {
      id: {},
      CDFILCENTPME: {},
      CDTIPOCARD: {},
      CDFILIAL: {},
      CDSERVFILI: {},
      CDCLIENTE: {},
      NMRAZSOCCLIE: {},
      DTEFETCLPME: {},
      QTCOMEFCLPME: {},
      QTFUNEFCLPME: {},
      QTDOAEFCLPME: {},
      NRORG: {},
      TOTAL: {},
      IDCLIENPRIN: {},
    },
  };
  return clientesDoServicoEfetivos;
};

const getServicosCardapioEfetivos = (): IRouteConfig<IExternalColumn> => {
  const filter = {
    CDFILIAL: localStorage.getItem('FILIAIS_IMPORTACAO') || '',
    DTINICIAL: localStorage.getItem('DATA_INICIAL') || '',
    DTFINAL: localStorage.getItem('DATA_FINAL') || '',
  };
  const servicosCardapioEfetivos: IRouteConfig<IExternalColumn> = {
    externalApi: true,
    persist: true,
    apiUrl: `${Config.endPoint}/servicosCardapioEfetivos?${qs.stringify(filter)}`,
    dataColumn: 'data',
    uniqueKey: 'id',
    columns: {
      id: {},
      CDTIPOCARD: {},
      NMTIPOCARD: {},
      DSCARDPME: {},
      CDFILCENTPME: {},
      NRCARDPME: {},
      NRCARDIAPME: {},
      CDSERVFILI: {},
      CDFILIAL: {},
      DATA_EXTENSO: {},
      DTCARDIAPME: {},
      IDCARDESPEC: {},
      NMSERVICO: {},
      QTCOMEFECPME: {},
      QTCLIEFECPME: {},
      QTFUNEFECPME: {},
      NRORG: {},
    },
  };
  return servicosCardapioEfetivos;
};

const getConfig = () => ({
  filial: getFilial(),
  inventarios: getInventarios(),
  produtos: getProdutos(),
  produtosInventario: getProdutosInventario(),
  af: getAF(),
  clientesDoServicoEfetivos: getClientesDoServicoEfetivos(),
  servicosCardapioEfetivos: getServicosCardapioEfetivos(),
});

export { getConfig };
