import { makeServer, reload, clearStorage } from '@zeedhi/zeedhi-miragejs';
import {
  Interceptor, Http, IDictionary, I18n, JsonCacheService,
} from '@zeedhi/core';
import adapter from '@zeedhi/axios-mirage-adapter';
import { DialogService, LoadingService } from '@zeedhi/common';
import { useMirage } from './use-mirage';
import { registerLogger, RequestLog, registerErrorLogger } from './request-logger';
import { getConfig } from './config';

Interceptor.addRequestSuccess((axiosConfig) => ({ ...axiosConfig, adapter }));

const endpoint = process.env.VUE_APP_MIRAGE_END_POINT;
let server = makeServer({
  endpoint, config: getConfig(), logging: true, passthrough: true,
});
const callMakeServer = (serverConfig: IDictionary) => {
  const p = new Promise((resolve) => {
    clearStorage();
    server.shutdown();
    server = makeServer({
      endpoint, config: serverConfig, logging: true, passthrough: true, persist: true,
    });
    // return server;
    resolve(server);
  });
  return p;
};

const executeExport = () => {
  const item = localStorage.getItem('request-log') || '[]';
  const logs = JSON.parse(item);
  LoadingService.show(' ');
  // para cada item do histórico, realiza uma request para o backend
  const pLogs = logs.map(async ({
    method, url, row, operation,
  }: RequestLog) => {
    if (method === 'post') {
      // Http.post('/manufacturers', row);
      if (operation === 'create') {
        await Http.post(`${url}/save`, { requestType: 'DataSet', dataset: [row] })
          .then((response: IDictionary) => {
            if (response.data?.length !== 0) {
              // const mensagem = response.data.error;
              registerErrorLogger(response);
            }
          }).catch((error) => {
            registerErrorLogger(error.response);
          });
      } else if (operation === 'delete') {
        await Http.post(`${url}/delete`, { requestType: 'DataSet', dataset: [row] });
      } else if (operation === 'update') {
        await Http.post(`${url}/update`, { requestType: 'DataSet', dataset: [row] });
      }
    }

    // const { id } = row;
    // if (method === 'put') {
    //   Http.put(`/manufacturers/${id}`, row);
    // }

    // if (method === 'delete') {
    //   Http.delete(`/manufacturers/${id}`);
    // }
  });
  return Promise.all(pLogs).then(async () => {
    JsonCacheService.clearJSONCache([
      { path: 'menu' },
      { path: 'zeedhi' },
    ]);
    await reload();
    // limpa o histórico
    localStorage.removeItem('request-log');
    LoadingService.hide();
    DialogService.hide();
  });
};

if (!navigator.onLine) {
  useMirage();
}

registerLogger();

window.addEventListener('online', async () => {
  useMirage(false);
  const hasImportedData = localStorage.getItem('hasImportedData');
  const item = localStorage.getItem('request-log') || '[]';
  const logs = JSON.parse(item);
  if (hasImportedData === 'true' && logs.length > 0) {
    DialogService.show({
      name: 'askForExport',
      type: 'warning',
      title: I18n.translate('scm::ATTENTION'),
      text: I18n.translate('scm::ONLINE_EXPORT'),
      buttons: [
        {
          name: 'yes',
          component: 'ZdButton',
          label: I18n.translate('scm::YES'),
          events: {
            click: () => {
              executeExport();
            },
          },
          outline: true,
        },
        {
          name: 'no',
          component: 'ZdButton',
          label: I18n.translate('scm::NO'),
          events: { click: () => DialogService.hide() },
        },
      ],
    });
  }
});

window.addEventListener('offline', async () => {
  useMirage();
});

export { callMakeServer, executeExport };
